@@ -62,6 +62,12 @@ cli_opts = [
                      'Can be supplied as "ipa-advertise-port" '
                      'kernel parameter.'),
 
+    cfg.StrOpt('advertise_protocol',
+               default=APARAMS.get('ipa-advertise-protocol', 'http'),
+               choices=['http', 'https'],
+               help='Protocol to use for the callback URL. HTTP is used by '
+                    'default, set to "https" if you have HTTPS configured.'),
+
     cfg.IntOpt('ip_lookup_attempts',
                min=1,
                default=int(APARAMS.get('ipa-ip-lookup-attempts', 6)),
@@ -119,10 +125,9 @@ cli_opts = [
 
     cfg.BoolOpt('standalone',
                 default=APARAMS.get('ipa-standalone', False),
-                help='Note: for debugging only. Start the Agent but suppress '
-                     'any calls to Ironic API. '
-                     'Can be supplied as "ipa-standalone" '
-                     'kernel parameter.'),
+                help='Start the Agent but suppress any calls to Ironic API, '
+                     'the agent runs on this mode for poll mode deployment. '
+                     'Can be supplied as "ipa-standalone" kernel parameter.'),
 
     cfg.StrOpt('inspection_callback_url',
                default=APARAMS.get('ipa-inspection-callback-url'),
