@@ -294,7 +294,8 @@ def _md_scan_and_assemble():
 
 def list_all_block_devices(block_type='disk',
                            ignore_raid=False,
-                           ignore_floppy=True):
+                           ignore_floppy=True,
+                           ignore_empty=True):
     """List all physical block devices
 
     The switches we use for lsblk: P for KEY="value" output, b for size output
@@ -310,6 +311,7 @@ def list_all_block_devices(block_type='disk',
                         devices and should be treated as such if encountered.
     :param ignore_floppy: Ignore floppy disk devices in the block device
                           list. By default, these devices are filtered out.
+    :param ignore_empty: Whether to ignore disks with size equal 0.
     :return: A list of BlockDevices
     """
 
@@ -378,12 +380,24 @@ def list_all_block_devices(block_type='disk',
         # Other possible type values, which we skip recording:
         #   lvm, part, rom, loop
         if devtype != block_type:
-            if (devtype is not None
-                    and any(x in devtype for x in ['raid', 'md'])
-                    and not ignore_raid):
+            if (devtype is None or ignore_raid):
+                LOG.debug("Skipping: {!r}".format(line))
+                continue
+            elif ('raid' in devtype
+                  and block_type in ['raid', 'disk']):
                 LOG.debug(
-                    "TYPE detected to contain 'raid' or 'md', signifying a "
+                    "TYPE detected to contain 'raid', signifying a "
                     "RAID volume. Found: {!r}".format(line))
+            elif (devtype == 'md'
+                  and (block_type == 'part'
+                       or block_type == 'md')):
+                # NOTE(dszumski): Partitions on software RAID devices have type
+                # 'md'. This may also contain RAID devices in a broken state in
+                # rare occasions. See https://review.opendev.org/#/c/670807 for
+                # more detail.
+                LOG.debug(
+                    "TYPE detected to contain 'md', signifying a "
+                    "RAID partition. Found: {!r}".format(line))
             else:
                 LOG.debug(
                     "TYPE did not match. Wanted: {!r} but found: {!r}".format(
@@ -403,6 +417,12 @@ def list_all_block_devices(block_type='disk',
             LOG.debug('Skipping RAM device %s', device)
             continue
 
+        # NOTE(dtantsur): some hardware represents virtual floppy devices as
+        # normal block devices with size 0. Filter them out.
+        if ignore_empty and not int(device['SIZE'] or 0):
+            LOG.debug('Skipping device %s with zero size', device)
+            continue
+
         name = os.path.join('/dev', device['KNAME'])
 
         try:
@@ -773,8 +793,9 @@ class HardwareManager(object, metaclass=abc.ABCMeta):
            'priority': the order steps will be run in. Ironic will sort all
                        the deploy steps from all the drivers, with the largest
                        priority step being run first. If priority is set to 0,
-                       the step will not be run during deployment, but may be
-                       run during zapping.
+                       the step will not be run during deployment
+                       automatically, but may be requested via deploy
+                       templates.
            'reboot_requested': Whether the agent should request Ironic reboots
                                the node via the power driver after the
                                operation completes.
@@ -1563,6 +1584,12 @@ class GenericHardwareManager(HardwareManager):
 
     def get_deploy_steps(self, node, ports):
         return [
+            {
+                'step': 'erase_devices_metadata',
+                'priority': 0,
+                'interface': 'deploy',
+                'reboot_requested': False,
+            },
             {
                 'step': 'apply_configuration',
                 'priority': 0,
@@ -1594,7 +1621,7 @@ class GenericHardwareManager(HardwareManager):
         self.validate_configuration(raid_config, node)
         if delete_existing:
             self.delete_configuration(node, ports)
-        self._do_create_configuration(node, ports, raid_config)
+        return self._do_create_configuration(node, ports, raid_config)
 
     def create_configuration(self, node, ports):
         """Create a RAID configuration.
@@ -1780,7 +1807,18 @@ class GenericHardwareManager(HardwareManager):
             utils.execute('mdadm', '--assemble', '--scan',
                           check_exit_code=False)
             raid_devices = list_all_block_devices(block_type='raid',
-                                                  ignore_raid=False)
+                                                  ignore_raid=False,
+                                                  ignore_empty=False)
+            # NOTE(dszumski): Fetch all devices of type 'md'. This
+            # will generally contain partitions on a software RAID
+            # device, but crucially may also contain devices in a
+            # broken state. See https://review.opendev.org/#/c/670807/
+            # for more detail.
+            raid_devices.extend(
+                list_all_block_devices(block_type='md',
+                                       ignore_raid=False,
+                                       ignore_empty=False)
+            )
             return raid_devices
 
         raid_devices = _scan_raids()
